from ezaws.utils.timing import (
    epoch_ms_to_date_time,
    convert_to_local,
    current_ms_after_epoch,
    datetime_to_epoch_in_ms,
    epoch_seconds_ago,
    epoch_minutes_ago,
    epoch_hours_ago,
    epoch_days_ago,
    date_seconds_ago,
    date_minutes_ago,
    date_days_ago,
    epoch_ms_to_date_time,
    epoch_to_date_time,
    convert_to_local,
)

import datetime


def test_current_ms_after_epoch():
    ms_after_epoch = current_ms_after_epoch()
    assert isinstance(ms_after_epoch, int)


def test_epoch_ms_to_date_time():
    date_time = epoch_ms_to_date_time(1)
    assert isinstance(date_time, datetime.datetime)


def test_datetime_to_epoch_in_ms():
    datetime_to_epoch_in_ms_result = datetime_to_epoch_in_ms(datetime.datetime.now())
    assert isinstance(datetime_to_epoch_in_ms_result, int)


def test_convert_to_local():
    date_time = convert_to_local(datetime.datetime.now())
    assert isinstance(date_time, datetime.datetime)


def test_epoch_to_date_time():
    epoch_to_date_time_return = epoch_to_date_time(1.1)
    assert isinstance(epoch_to_date_time_return, datetime.datetime)


def test_xxx_ago():
    test_func = [
        epoch_seconds_ago,
        epoch_minutes_ago,
        epoch_hours_ago,
        epoch_days_ago,
        date_seconds_ago,
        date_minutes_ago,
        date_days_ago,
    ]
    for f in test_func:
        assert isinstance(f(1), int)



from ezaws import Log
from pprint import pprint

if __name__ == "__main__":

    cw_log = Log(name="cloudwatch_example", region="eu-central-1")
    resp = cw_log.create_log_group()
    pprint(resp)
    resp = cw_log.create_stream(stream_name="general")
    pprint(resp)
    resp = cw_log.get_log_streams()
    pprint(resp)



from ezaws.ssm.parameter_store import ParameterStore
from ezaws.models.ssm import CreateParameter
import time

to_create = {
    "DataType": "text",
    "Name": "/passwords/infrastructure/created_parameter",
    "Type": "String",
    "Value": "SECRET_INFORMATION",
    "Version": 1,
    "Tier": "Standard",
    "Description": "Created with CreateParameter",
    "Overwrite": True,
}
param_to_create = CreateParameter(**to_create)

if __name__ == "__main__":
    from pprint import pprint

    ps = ParameterStore(region="eu-central-1")

    print("Creating parameter")
    ps.create_parameter(parameter=param_to_create)
    time.sleep(3)
    print("Retrieve parameter")
    param_resp = ps.get_parameter("/passwords/infrastructure/created_parameter")
    pprint(param_resp)
    print("Retrieve all parameters")
    region_param_resp = ps.describe_region_parameters()
    for param in region_param_resp:
        pprint(param.Name)

    print("Deleting parameter")
    ps.delete_parameter("/passwords/infrastructure/created_parameter")
    time.sleep(3)
    print("Retrieve all parameters")
    region_param_resp = ps.describe_region_parameters()
    for param in region_param_resp:
        pprint(param.Name)



from ezaws import S3Bucket, Region

from pprint import pprint

BUCKET_NAME = "cicdtestingbucket"
if __name__ == "__main__":
    bucket = S3Bucket(region=Region.eu_central_1, name=BUCKET_NAME)

    print("CREATE BUCKET\n\n")
    resp = bucket.create_bucket()
    pprint(resp)



from ezaws import Messenger
from ezaws.models.sqs import QueueAttributes, PurgeQueueResponse
from ezaws.sqs.functions import list_queues
from uuid import uuid4


QUEUE_NAME = str(uuid4()) + "_testing_queue_do_not_delete"
REGION = "eu-central-1"


def test_messenger_instantiation_from_name():
    msgnr = Messenger.from_queue_name(queue_name=QUEUE_NAME, region=REGION)
    assert isinstance(msgnr, Messenger)
    print(QUEUE_NAME, REGION)
    resp = msgnr._get_queue_url(queue_name=QUEUE_NAME, region_name=REGION)
    assert isinstance(resp, str)
    resp = msgnr.purge_queue()
    assert isinstance(resp, PurgeQueueResponse)
    msgnr.delete_queue()
    qs = list_queues(region=REGION)
    assert QUEUE_NAME not in qs.QueueUrls


def test_messenger_instantiation(test_q):
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    assert isinstance(msgnr, Messenger)


def test_send_messages(test_q):
    """Instantiate a messenger and send a message"""
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    msg_resp = msgnr.send_message(message="test_message")

    assert isinstance(msg_resp.MD5OfMessageBody, str)


def test_read_messages(test_q):
    """Instantiate a messenger and read a message"""
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    q_msgs = msgnr.read_messages(nr_of_messages=10)
    assert isinstance(q_msgs.Messages, list)


def test_delete_message(test_q):
    """Instantiate a messenger and read a message"""
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    msgnr.send_message(message="msg")
    q_msgs = msgnr.read_messages(nr_of_messages=10)
    while len(q_msgs.Messages) == 0:
        q_msgs = msgnr.read_messages(nr_of_messages=10)
    resp = msgnr.delete_message(receipt_handle=q_msgs.Messages[0]["ReceiptHandle"])
    assert resp.ResponseMetadata["HTTPStatusCode"] == 200


def test_get_queue_count(test_q):
    """Instantiate a messenger and read a message"""
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    q_msgs = msgnr.get_queue_count()

    assert isinstance(q_msgs, int)


def test_get_queue_attributes(test_q):
    msgnr = Messenger(
        queueu_url=test_q.url,
        region=test_q.region,
    )
    q_attr = msgnr.get_queue_attributes()
    assert isinstance(q_attr, QueueAttributes)
    assert q_attr.DelaySeconds == "0"
    assert q_attr.VisibilityTimeout == "30"
    assert "testing_queue_do_not_delete" in q_attr.QueueArn



from ezaws.sqs.functions import list_queues
from uuid import uuid4
from ezaws import Messenger
from collections import namedtuple
import pytest

TestQ = namedtuple("TestQ", "url region")


@pytest.fixture(scope="session")
def test_q() -> TestQ:
    """returns a q name that can be used for testing"""

    region = "eu-central-1"
    qs = list_queues(region=region)
    for q_url in qs:
        if "_testing_queue_do_not_delete" in q_url:
            return TestQ(q_url, region)
    queue_name = str(uuid4()) + "_testing_queue_do_not_delete"
    msgnr = Messenger.from_queue_name(queue_name=queue_name, region=region)
    return TestQ(msgnr.queueu_url, region)



from ezaws import Messenger
from pprint import pprint
from ezaws import Region

if __name__ == "__main__":

    msgnr = Messenger.from_queue_name(
        queue_name="example-queue", region=Region.eu_central_1
    )

    msg = msgnr.send_message("message 1")

    pprint(msg)
    q_count = msgnr.get_queue_count()
    print(q_count)
    pprint(vars(msgnr.get_queue_attributes()))

    """
    print(msgnr.get_queue_count())
    print(msgnr.get_queue_attributes())


    msg = msgnr.send_message("message 3")
    msg = msgnr.send_message("message 4")
    msg = msgnr.send_message("message 5")

    msg_read = msgnr.read_messages(nr_of_messages=10)

    for m in msg_read:
        m = Message(**m)
        print(f"read message {m.Body}")
        msgnr.delete_message(receipt_handle=m.ReceiptHandle)


    """
    """

for m in msg_read:
    m = Message(**m)
    print(f"read message {m.Body}")
    msgnr.delete_message(receipt_handle=m.ReceiptHandle)
    """
    """
    qs = list_queues(region="eu-central-1")
    pprint(qs)

    
    for q in qs.QueueUrls:
        print(q)
        if "testing_queue_do_not_delete" in q:
            delete_queue(queue_name=q, region="eu-central-1")
    pprint(list_queues(region="eu-central-1"))
    
    queue_name = "example-queue"
    create_queue_resp = create_queue(queue_name)
    pprint(create_queue_resp)
    queue_url_resp = get_queue_url(queue_name)
    print("get queue response")
    pprint(queue_url_resp)
    val = input("Type 'delete' in case you want to delete the 'example-queue':")
    if val == "delete":
        delete_queu_response = delete_queue(queue_name)
        print("delete queue response")
        pprint(delete_queu_response)
    else:
        print("'example-queue' is not deleted.")
    queueu_url = "https://eu-central-1.queue.amazonaws.com/717687450252/example-queue"
    region_name = "eu-central-1"

    print("Purging queue:")
    pugre_response = purge_queue(queueu_url=queueu_url, region_name=region_name)
    pprint(pugre_response)
    queueu_url = "https://eu-central-1.queue.amazonaws.com/717687450252/example-queue"
    region_name = "eu-central-1"
    print("sending message:")
    send_response = send_message(
        queueu_url=queueu_url,
        region_name=region_name,
        message='{"important-key":"important-value-1"}',
    )
    pprint(send_response)
    time.sleep(2)
    receive_response = receive_message(
        queueu_url=queueu_url, region_name=region_name, nr_of_messages=10
    )

    send_message_id = send_response["MessageId"]
    send_message_md5 = send_response["MD5OfMessageBody"]
    print(
        f"\n\n\nMessage that was send to the queue has id {send_message_id} and md5 {send_message_md5}.\n\n"
    )

    to_delete = []
    print("Messages in the queue:")
    for msg in receive_response["Messages"]:
        pprint(msg)
        to_delete.append(msg["ReceiptHandle"])
    print(f"Deleting messages with the following receipt handles:")
    for item in to_delete:
        print(item)
    for receipt_handle in to_delete:

        delete_response = delete_message(
            queueu_url=queueu_url,
            region_name=region_name,
            receipt_handle=receipt_handle,
        )
        pprint(delete_response)
    """



from ezaws.models.dynamodb import Table, GetItemResponse


def test_table_creation():
    table = Table(table_name="Humans", rcu=1, wcu=1)
    table.add_attribute(attribute_name="name", attribute_type="S", key_type="RANGE")
    table.add_attribute(attribute_name="id", attribute_type="N", key_type="HASH")
    assert isinstance(table, Table)


def test_GetItemResponse():
    get_item_response = GetItemResponse(
        **{
            "Item": {"id": "1828", "name": "Arnetta"},
            "ResponseMetadata": {
                "HTTPHeaders": {
                    "connection": "keep-alive",
                    "content-length": "51",
                    "content-type": "application/x-amz-json-1.0",
                    "date": "Tue, 31 May 2022 06:43:40 GMT",
                    "server": "Server",
                    "x-amz-crc32": "859391921",
                    "x-amzn-requestid": "KNCSVK20TCKSJSVFJQQTCVR647VV4KQNSO5AEMVJF66Q9ASUAAJG",
                },
                "HTTPStatusCode": 200,
                "RequestId": "KNCSVK20TCKSJSVFJQQTCVR647VV4KQNSO5AEMVJF66Q9ASUAAJG",
                "RetryAttempts": 0,
            },
        }
    )
    assert get_item_response



from ezaws import S3Bucket, Region
import time
from pprint import pprint

BUCKET_NAME = "s3examplebucketusingezaws"
if __name__ == "__main__":
    bucket = S3Bucket(region=Region.eu_central_1, name=BUCKET_NAME)

    print("CREATE BUCKET\n\n")
    resp = bucket.create_bucket()
    pprint(resp)

    print("UPLOAD FILE\n\n")
    resp = bucket.upload_file("example.txt", "some_file")
    pprint(resp)

    time.sleep(2)

    print("GET OBJECT METADATA\n\n")
    resp = bucket.get_object_metadata(s3_file_name="some_file")
    pprint(resp)

    print("DELETE FILE\n\n")
    resp = bucket.delete_object(s3_key_name="some_file")
    pprint(resp)

    print("DELETE BUCKET\n\n")
    resp = bucket.delete_bucket()
    pprint(resp)



from ezaws import Lambda
from ezaws import Region
from ezaws.models.lambdas import ListFunctionsResponse


def test_list_functions():

    lambda_handler = Lambda(region=Region.eu_central_1)

    resp = lambda_handler.list_functions()
    assert isinstance(resp, ListFunctionsResponse)



from ezaws import S3, S3Bucket, TCBuilder
from ezaws.models.s3 import (
    ListBucketsResponse,
    ObjectMetadata,
    VersioningResponse,
    DeleteObjectResponse,
)
from ezaws import Region
from uuid import uuid4
import pathlib

path = pathlib.Path(__file__).parent.as_posix()
TESTING_FILE = path + "/testing_file.txt"
S3_KEY_NAME = str(uuid4()) + "_testing_key"
S3_KEY_NAME_PUT = str(uuid4()) + "_testing_key"
S3_KEY_NAME_CONTROLLED = str(uuid4()) + "_testing_key"


def test_s3_class():
    """instantiate S3 class and list all buckets"""
    s3 = S3()
    resp = s3.list_buckets()
    assert isinstance(resp, ListBucketsResponse)


def test_s3_bucket():
    """instantiate S3Bucket and check versioning information"""
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")
    resp = bucket.get_versioning()
    assert isinstance(resp, VersioningResponse)


def test_upload():
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")

    bucket.upload_file(
        file_name=TESTING_FILE,
        s3_key_name=S3_KEY_NAME,
    )
    resp = bucket.get_object_metadata(s3_file_name=S3_KEY_NAME)
    # only gives a valid return in case the file exists
    assert isinstance(resp, ObjectMetadata)


def test_get_filesize():
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")
    size_in_mb = bucket.get_file_size(s3_file_name=S3_KEY_NAME)
    size_in_b = bucket.get_file_size(s3_file_name=S3_KEY_NAME, block_size="bytes")
    assert isinstance(size_in_mb, int)
    assert isinstance(size_in_b, int)


def test_delete_key():
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")
    resp = bucket.delete_object(s3_key_name=S3_KEY_NAME)
    assert isinstance(resp, DeleteObjectResponse)


def test_put():
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")
    bucket.put_object(file_name=TESTING_FILE, s3_key_name=S3_KEY_NAME_PUT)
    resp = bucket.delete_object(s3_key_name=S3_KEY_NAME_PUT)
    assert isinstance(resp, DeleteObjectResponse)


def test_controlled():
    bucket = S3Bucket(region=Region.eu_central_1, name="cicdtestingbucket")
    bucket.controlled_upload(file_name=TESTING_FILE, s3_key_name=S3_KEY_NAME_CONTROLLED)
    resp = bucket.delete_object(s3_key_name=S3_KEY_NAME_CONTROLLED)
    assert isinstance(resp, DeleteObjectResponse)

    tcbuilder = TCBuilder(use_threads=True, max_concurrency=5)
    bucket.controlled_upload(
        file_name=TESTING_FILE, s3_key_name=S3_KEY_NAME_CONTROLLED, tc=tcbuilder
    )
    resp = bucket.delete_object(s3_key_name=S3_KEY_NAME_CONTROLLED)
    assert isinstance(resp, DeleteObjectResponse)

    tc_dict = {"use_threads": True, "max_concurrency": 5}
    bucket.controlled_upload(
        file_name=TESTING_FILE, s3_key_name=S3_KEY_NAME_CONTROLLED, tc=tc_dict
    )
    resp = bucket.delete_object(s3_key_name=S3_KEY_NAME_CONTROLLED)
    assert isinstance(resp, DeleteObjectResponse)



from ezaws import S3Bucket, Region
import time
from pprint import pprint

BUCKET_NAME = "s3examplebucketusingezaws"

if __name__ == "__main__":
    bucket = S3Bucket(region=Region.eu_central_1, name=BUCKET_NAME)
    print("DELETE BUCKET\n\n")
    resp = bucket.delete_bucket()
    pprint(resp)



from ezaws import Log, LogEvent
from pprint import pprint
import time
from uuid import uuid4


if __name__ == "__main__":

    cw_log = Log(name="cloudwatch_example", region="eu-central-1")

    # ez log:
    resp = cw_log.log(message="using log")
    pprint(resp)
    resp = cw_log.log_messages(
        messages=["using log_messages", "using log_messages again"],
    )
    pprint(resp)
    # log events

    events = [
        {
            "timestamp": int(round(time.time() * 1000)),
            "message": "event-1",
        },
        {
            "timestamp": int(round(time.time() * 1000)),
            "message": "even-2",
        },
        LogEvent(message="event-3", timestamp=int(round(time.time() * 1000))),
    ]
    resp = cw_log.log_events(events=events)
    pprint(resp)
    messages = []
    for i in range(100):
        messages.append("unique message" + str(uuid4()))
    cw_log.log_messages(
        messages=messages,
    )



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB
import logging

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.INFO)

if __name__ == "__main__":

    ddb = DynamoDB(region=Region.eu_central_1)
    pprint(ddb.partiql_query("SELECT * FROM Humans"))
    pprint(ddb.partiql_query("SELECT name FROM Humans WHERE name ='x'"))
    pprint(ddb.partiql_query("SELECT name FROM Humans WHERE id =1"))
    pprint(ddb.partiql_query("SELECT * FROM Humans WHERE id =7"))



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB

if __name__ == "__main__":
    ddb = DynamoDB(region=Region.eu_central_1)
    del_respone = ddb.delete_table(table_name="Humans")
    pprint(del_respone)



from pprint import pprint
from ezaws import Region
from ezaws import DynamoDB
import logging

logging.basicConfig(format="%(levelname)s:%(message)s", level=logging.INFO)

if __name__ == "__main__":

    ddb = DynamoDB(region=Region.eu_central_1)
    result = ddb.get_item(table_name="Humans", item={"id": 1828, "name": "Arnetta"})

    pprint(result)



from ezaws import Lambda


if __name__ == "__main__":
    from pprint import pprint

    # lambda_handler = Lambda(region=Region.eu_central_1)
    lambda_handler = Lambda()
    resp = lambda_handler.list_functions()
    pprint(resp)
    for func in resp:
        print(func.FunctionName)

    resp = lambda_handler.run_function(
        function_name="klundert-lambda-sam-helloworldpython3-E5pK6x3FSUfk",
        log_type=None,
    )
    pprint(resp.dict(), width=5)
    print(resp.LogResult)
    print(resp.payload_to_str())



from ezaws import Log
from pprint import pprint
import datetime
import time

if __name__ == "__main__":

    cw_log = Log(name="cloudwatch_example", region="eu-central-1")
    # grab everything
    resp = cw_log.get_log_events(limit=5000)
    pprint(resp)

    for event in resp.events:
        print(event.message)

        print(datetime.datetime.utcfromtimestamp(event.timestamp / 1000.0))

    # the last n minutes:
    print("Getting the last minutes.")
    resp = cw_log.get_log_events_last_minutes(minutes=1)
    pprint(resp)
    print("Getting the last hours.")
    resp = cw_log.get_log_events_last_hours(hours=2)
    pprint(resp)
    print("Getting the last seconds.")
    cw_log.log(message="just a few seconds ago")
    time.sleep(1)
    resp = cw_log.get_log_events_last_seconds(seconds=5)
    pprint(resp)
    for event in resp.events:
        # The Event has several fields/properties:
        # - message: the message that was logged to Cloudwatch
        # - timestamp: the timestap that is recorded in Cloudwatch:
        #       epoch in ms, which is the number of milliseconds
        #        after Jan 1, 1970
        # - date_time_local: the timestap converted to a
        #       datetime.datetime instance that represents the
        #        local time.
        # - epoch_local: date_time_local converted to the epoch in ms.
        print(event.message)
        print(event.timestamp)
        print(event.date_time_local)
        print(event.epoch_local)
